package new_mobile_api_controllers

import (
	"XT_New/enums"
	"XT_New/models"
	"XT_New/service"
	"strconv"
	"strings"
	"time"
)

type NewRoleApiController struct {
	NewMobileBaseAPIAuthController
}

func (this *NewRoleApiController) GetAllOrgUser() {

	adminUserInfo := this.GetMobileAdminUserInfo()
	viewModels, _, _ := service.GetAllAdminUsersAndRole(adminUserInfo.Org.Id, adminUserInfo.App.Id, 1, 10)
	this.ServeSuccessJSON(map[string]interface{}{
		"admins": viewModels,
	})
}

func (this *NewRoleApiController) EditAdmin() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	adminUserId, _ := this.GetInt64("uid")
	name := this.GetString("name")
	userTitle := this.GetString("title")
	roleIds := this.GetString("role_ids")
	if adminUserId <= 0 || len(name) == 0 || len(roleIds) <= 0 {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeParamWrong)
		return
	}
	appRole, getAppRoleErr := service.GetAppRole(adminUserInfo.Org.Id, adminUserInfo.App.Id, adminUserId)
	if getAppRoleErr != nil {
		//beego.Error("查询管理员信息时失败:", getAppRoleErr)
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	}
	if appRole == nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeAdminUserNotExist)
		return
	}
	appRole.UserName = name
	appRole.UserTitleName = userTitle
	appRole.RoleIds = roleIds
	appRole.ModifyTime = time.Now().Unix()
	saveErr := service.SaveAppRole(appRole)
	if saveErr != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDBUpdate)
	} else {
		this.ServeSuccessJSON(nil)
	}
}

func (this *NewRoleApiController) GetEditAdminInitData() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	roles, _ := service.GetAllOrgValidRoles(adminUserInfo.Org.Id)
	this.ServeSuccessJSON(map[string]interface{}{
		"roles": roles,
	})
}

func (this *NewRoleApiController) GetAdminUserInfo() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	adminUserId, _ := this.GetInt64("uid")
	appRole, getAppRoleErr := service.GetAppRole(adminUserInfo.Org.Id, adminUserInfo.App.Id, adminUserId)
	if getAppRoleErr != nil {
		//beego.Error("查询管理员信息时失败:", getAppRoleErr)
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	}
	if appRole == nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeAdminUserNotExist)
		return
	}

	this.ServeSuccessJSON(map[string]interface{}{
		"user_info": appRole,
	})

}

func (this *NewRoleApiController) StopAdminUser() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	adminUserId, _ := this.GetInt64("uid")
	appRole, getAppRoleErr := service.GetAppRole(adminUserInfo.Org.Id, adminUserInfo.App.Id, adminUserId)
	if getAppRoleErr != nil {
		//beego.Error("查询管理员信息时失败:", getAppRoleErr)
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	}
	if appRole == nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeAdminUserNotExist)
		return
	}

	enable, _ := this.GetBool("enable")

	if enable {
		appRole.Status = 1
	} else {
		appRole.Status = 0
	}
	appRole.ModifyTime = time.Now().Unix()
	saveErr := service.SaveAppRole(appRole)
	if saveErr != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDBUpdate)
	} else {
		this.ServeSuccessJSON(nil)
	}

}

func (this *NewRoleApiController) CreateAdminUser() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	mobile := this.GetString("mobile")
	name := this.GetString("name")
	role_ids := this.GetString("role_ids")
	userTitle := this.GetString("title")

	if len(mobile) == 0 || len(name) == 0 || len(role_ids) <= 0 {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeParamWrong)
		return
	}

	// 判断是否已存在该手机号
	if adminUser, err := service.GetValidAdminUserByMobileReturnErr(mobile); err != nil {
		//beego.Error("查询用户是否已被添加为管理员时失败:", err)
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	} else {
		if adminUser == nil { //新增账号和用户
			_, password, createErr := service.CreateGeneralAdminUser(adminUserInfo.Org.Id, adminUserInfo.App.Id, mobile, name, userTitle, role_ids)
			if createErr != nil {
				//beego.Error("创建管理员失败:", createErr)
				this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDBCreate)
				return

			} else {
				sendSMSErr := service.SMSSendInviteMobileToJoinOrgAdmin(name, mobile, password)
				if sendSMSErr != nil {
				}

				this.ServeSuccessJSON(nil)
				return
			}
		} else {

			total, _ := service.FindAdminUserByID(adminUser.Id, adminUserInfo.Org.Id)

			if total <= 0 {
				//新增用户
				app_role := &models.App_Role{
					AdminUserId:   adminUser.Id,
					OrgId:         adminUserInfo.Org.Id,
					AppId:         adminUserInfo.App.Id,
					Avatar:        "",
					UserName:      name,
					UserTitleName: userTitle,
					Status:        1,
					CreateTime:    time.Now().Unix(),
					ModifyTime:    time.Now().Unix(),
					RoleIds:       role_ids,
				}
				err := service.CreateUserRole(app_role)
				if err != nil {
					this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDBCreate)
					return
				}
				this.ServeSuccessJSON(nil)
			} else {
				this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeRepeatCreateStaffException)
				return
			}

			return
		}
	}

}

func (this *NewRoleApiController) GetAllOrgRole() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	roles, err := service.GetAllOrgValidRoles(adminUserInfo.Org.Id)
	if err != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	} else {
		this.ServeSuccessJSON(map[string]interface{}{
			"roles": roles,
		})
	}

}

func (this *NewRoleApiController) EditRole() {
	role_id, _ := this.GetInt64("id", 0)
	desc := this.GetString("desc")

	role, _ := service.GetRoleByRoleID(role_id)

	role.RoleIntro = desc
	role.ModifyTime = time.Now().Unix()

	err := service.SaveRole(role)
	if err != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
	}
	this.ServeSuccessJSON(map[string]interface{}{
		"role": role,
	})
	return
}

func (this *NewRoleApiController) GetOrgRoleInfo() {
	role_id, _ := this.GetInt64("id", 0)
	role, err := service.GetRoleByRoleID(role_id)
	if err != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	} else {
		this.ServeSuccessJSON(map[string]interface{}{
			"role": role,
		})
	}

}

func (this *NewRoleApiController) CreateRole() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	role_name := this.GetString("name")
	role_desc := this.GetString("desc")

	role := &models.Role{
		RoleName:     role_name,
		RoleIntro:    role_desc,
		Creator:      adminUserInfo.AdminUser.Id,
		OrgId:        adminUserInfo.Org.Id,
		AppId:        adminUserInfo.App.Id,
		Status:       1,
		IsSuperAdmin: false,
		CreateTime:   time.Now().Unix(),
		ModifyTime:   time.Now().Unix(),
	}

	err := service.CreateOrgRole(role)
	if err != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	} else {
		this.ServeSuccessJSON(map[string]interface{}{
			"role": role,
		})
	}

}

func (this *NewRoleApiController) EditRolePermission() {
	role_id, _ := this.GetInt64("id", 0)
	purview_id := this.GetString("purview_id")
	is_open, _ := this.GetInt64("type", 0)

	permissions, err := service.GetRolePurviewIds(role_id)
	if err != nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	} else {

		if is_open == 1 { //打开
			permission_arr := strings.Split(permissions, ",")
			permission_arr = append(permission_arr, purview_id)
			permissions = strings.Join(permission_arr, ",")
			//插入数据库
			purview, _ := service.GetRolePurview(role_id)
			purview.PurviewIds = permissions
			err := service.SaveRolePurview(purview)
			if err != nil {
				this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
				return
			}

			this.ServeSuccessJSON(map[string]interface{}{
				"is_open": 1,
			})

		} else { //关闭

			permission_arr := strings.Split(permissions, ",")
			for index, id := range permission_arr {
				if id == purview_id {
					permission_arr = append(permission_arr[:index], permission_arr[index+1:]...)
				}
			}
			purview, _ := service.GetRolePurview(role_id)
			purview.PurviewIds = strings.Join(permission_arr, ",")

			err := service.SaveRolePurview(purview)
			if err != nil {
				this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
				return
			}

			this.ServeSuccessJSON(map[string]interface{}{
				"is_open": 0,
			})

		}
	}
}

func (this *NewRoleApiController) GetAllPermission() {
	module, _ := this.GetInt64("module")
	parent_id, _ := this.GetInt64("id")
	purviews, _ := service.GetAllPurview(module, parent_id)
	this.ServeSuccessJSON(map[string]interface{}{
		"purviews": purviews,
	})
}

func (this *NewRoleApiController) DeleteOrgRole() {
	roleID, _ := this.GetInt64("role_id")
	enable, _ := this.GetBool("enable")
	if roleID <= 0 {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeParamWrong)
		return
	}

	adminUserInfo := this.GetMobileAdminUserInfo()

	role, getRoleErr := service.GetRoleByRoleID(roleID)
	if getRoleErr != nil {
		//beego.Error("获取角色失败:", getRoleErr)
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDataException)
		return
	} else if role == nil {
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeRoleNotExist)
		return
	}

	if enable == false {
		if count, _ := service.RoleAdminUserCountTwo(adminUserInfo.Org.Id, adminUserInfo.App.Id, roleID); count != 0 {
			this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeCannotRemoveRole)
			return
		}
	}

	if enable {
		role.Status = 1
	} else {
		role.Status = 2
	}
	role.ModifyTime = time.Now().Unix()
	saveErr := service.ModifyRole(role)
	if saveErr != nil {
		//beego.Error("修改角色失败:", role.Id, saveErr)
		this.ServeFailJSONWithSGJErrorCode(enums.ErrorCodeDBUpdate)
	} else {
		this.ServeSuccessJSON(nil)
	}

}

func (this *NewRoleApiController) GetRolePurviews() {
	adminUserInfo := this.GetMobileAdminUserInfo()
	role_id, _ := this.GetInt64("id", 0)
	rolePurview, _ := service.GetRoleAndPurviewById(role_id, adminUserInfo.Org.Id, adminUserInfo.App.Id)
	this.ServeSuccessJSON(map[string]interface{}{
		"role_purview": rolePurview,
	})
}

func (this *NewRoleApiController) AddRoleStaff() {
	//adminUserInfo := this.GetMobileAdminUserInfo()
	role_id, _ := this.GetInt64("id", 0)
	staff_ids := this.GetString("ids")
	ids := strings.Split(staff_ids, ",")
	for _, item := range ids {
		id, _ := strconv.ParseInt(item, 10, 64)
		role, _ := service.FindAdminUserID(id)
		role.RoleIds = role.RoleIds + "," + strconv.FormatInt(role_id, 10)
		service.SaveAdminUser(&role)
	}
	this.ServeSuccessJSON(map[string]interface{}{
		"msg": "添加成功",
	})

}
